/*****************************************************************************
* Copyright (c) Future Technology Devices International 2014
* propriety of Future Technology devices International.
*
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by FTDI. 
* FTDI accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file                           SC16IS760IBS.h
* @brief                          Contains Arduino specific macros for SC16IS760IBS hardware.
								  Tested platform version: Arduino 1.0.5.
* @version                        1.0.0
* @date                           2014/10/01
*
*/

#ifndef _SC16IS760IBS_H_
#define _SC16IS760IBS_H_


/* Macros for buffer information */
#define FT_VI800A_MAX_FIFO 64
#define FT_VI800A_MAX_FIFO_TRIGGER_LEVEL 60
#define FT_VI800A_MIN_FIFO_TRIGGER_LEVEL 4

#define FT_VI800A_MAX_FREQUENCY 14745600  

#define FT_VI800A_MIN_BAUD_RATE 15
#define FT_VI800A_MAX_BAUD_RATE 921600

/* Macros for registers addresses */
#define FT_VI800A_THR        0x00 << 3
#define FT_VI800A_RHR        0x00 << 3
#define FT_VI800A_IER        0x01 << 3
#define FT_VI800A_FCR        0x02 << 3
#define FT_VI800A_IIR        0x02 << 3
#define FT_VI800A_LCR        0x03 << 3
#define FT_VI800A_MCR        0x04 << 3  
#define FT_VI800A_LSR        0x05 << 3
#define FT_VI800A_MSR        0x06 << 3
#define FT_VI800A_SPR        0x07 << 3
#define FT_VI800A_TXLVL      0x08 << 3
#define FT_VI800A_RXLVL      0x09 << 3
#define FT_VI800A_DLAB       0x80 << 3
#define FT_VI800A_IODIR      0x0A << 3
#define FT_VI800A_IOSTATE    0x0B << 3
#define FT_VI800A_IOINTMSK   0x0C << 3
#define FT_VI800A_IOCTRL     0x0E << 3
#define FT_VI800A_EFCR       0x0F << 3

//These registers can only be accessed when MCR[2]=1 and EFR[4]=1
#define FT_VI800A_TCR			0x06 << 3
#define FT_VI800A_TLR			0x07 << 3

//These registers can only be accessed when the LCR[7] is logic 1.
#define FT_VI800A_DLL        0x00 << 3  
#define FT_VI800A_DLH        0x01 << 3

//These registers can only be accessed when the LCR is set to 1011 1111b(0xBF)
#define FT_VI800A_EFR        0x02 << 3
#define FT_VI800A_XON1       0x04 << 3  
#define FT_VI800A_XON2       0x05 << 3
#define FT_VI800A_XOFF1      0x06 << 3
#define FT_VI800A_XOFF2      0x07 << 3



//The EFR register enables or disables the enhanced features of the UART.
#define FT_EFR_ENABLE_CTS_FLOW_CONTROL 0x80
#define FT_EFR_ENABLE_RTS_FLOW_CONTROL 0x40
#define FT_EFR_ENABLE_SPECIAL_CHAR_DETECTION 0x20
#define FT_EFR_ENABLE_ENHANCED_FUNCTIONS 0x10
#define FT_EFR_SOFTWARE_FLOW_CONTROL_COMBINATION 0x0F

#define FT_EFR_SW_FLOW_MASK 0xF0
#define FT_EFR_HW_FLOW_MASK 0xC0
#define FT_EFR_DISABLE_RS485_FEATURES 0x86
#define FT_EFR_RESET_FLOW_FEATURES

//These features are enabled in the FCR register
#define FT_FCR_RX_TRIGGER_LEVEL 0xC0
#define FT_FCR_TX_TRIGGER_LEVEL 0x30
#define FT_FCR_TX_FIFO_RESET 0x04
#define FT_FCR_RX_FIFO_RESET 0x02
#define FT_FCR_ENABLE_TX_RX_FIFO 0x01
#define FT_FCR_Trigger_Mask 0x0F

//The IER register enables each of the six types of interrupts
#define FT_IER_ENABLE_CTS_INTERRUPT 0x80  //also see NOTE 2
#define FT_IER_ENABLE_RTS_INTERRUPT 0x40  //also see NOTE 2
#define FT_IER_XOFF_INTERRUPT 0x20  //also see NOTE 2
#define FT_IER_SLEEP_MODE 0x10 //Note an interrupt this bit enables the sleep mode. Also see NOTE 2.
#define FT_IER_MODEM_STATUS_INTERRUPT 0x08  //this bit only enables the modem status register interrupt
#define FT_IER_RECEIVE_LINE_STATUS_INTERRUPT 0x04
#define FT_IER_THR_EMPTY_INTERRUPT 0x02
#define FT_IER_RX_DATA_AVAILABLE_INTERRUPT 0x01

//The IIR register states the source of the interrupt(s).  The first bit of this register indicates whether there is a pending interrupt or not, IIR[5:1] specifies the 5-bit encoded interrupt and IIR[7:6] is the mirror of the contents of FCR[0].  
/*
  //An interrupt is pending only if the first bit is a logic 0
#define FT_IIR_RECEIVER_LINE_STATUS_ERROR 0x06
#define FT_IIR_RECEIVER_TIME_OUT_INTERRUPT 0x0C
#define FT_IIR_RHR_INTERRUPT 0x04
#define FT_IIR_THR_INTERRUPT 0x02
#define FT_IIR_MODEM_INTERRUPT 0x00 //modem interrupt status is reflected by reading the MSR register
#define FT_IIR_INPUT_PIN_CHANGE_OF_STATE 0x30 //GPIO interrupt status is reflected by reading the IOState register
#define FT_IIR_RECEIVED_SPECIAL_CHARACTER 0x10
#define FT_IIR_CTS_RTS_CHANGED_TO_HIGH_STATE 0x20
*/
#define FT_IIR_INTERRUPT_IS_PENDING 0x01
#define FT_IIR_ALL_INTERRUPTS 0x3F

//THE LCR register determines the data format such as word length, number of stop bits and parity type.  Refer to the relevant tables for the complete lists of the data formats.
#define FT_LCR_ENABLE_DIVISOR_LATCH 0x80
#define FT_LCR_LINE_BREAK_CONDITION 0x40 //logic 0 = no TX break condition(default). logic 1 = forces the transmitter output (TX) to a logic 0 to alert the communication terminal to a line break condition.
#define FT_LCR_SET_PARITY_BIT 0x20
#define FT_LCR_PARITY_TYPE_SELECT 0x10
#define FT_LCR_PARITY_ENABLE 0x08
#define FT_LCR_NUMBER_OF_STOP_BITS 0x04
#define FT_LCR_WORD_LENGTH 0x03
#define FT_LCR_ACCESS_ENHANCED_REGISTERS 0xBF
#define FT_LCR_RESET_DEFAULT_VALUE 0x1D
#define FT_LCR_DATAFORMAT_MASK 0xC0

//The MCR register controls the interface with the mode, data set, or peripheral device that is emulating the modem.
#define FT_MCR_CLOCK_DIVISOR 0x80 //0=divide by 1 clock input, 1=divide by 4 clock input
#define FT_MCR_ENABLE_IRDA_MODE 0x40 //0=uart mode(default) 1=IrDA mode
#define FT_MCR_ENABLE_XON_ANY_FUNCTION 0x20
#define FT_MCR_ENABLE_INTERNAL_LOOPBACK 0x10
#define FT_MCR_ENABLE_TCR_TLR_REGISTERS 0x04
#define FT_MCR_FORCE_RTS_TO_ACTIVE 0x02 //force the rts line to become active
#define FT_MCR_FORCE_DTR_TO_ACTIVE 0x01  //if GPIO5 is selected as DRT modem pin through IOControl register bit 1 or (macro name here)

//The LSR register shows various status on the receiver.  NOTE: break interrupt, framing error, and parity error are indicating the condition of the character at the top of the RX FIFO(the next character to be read).  Errors in a character are identified by reading the LSR and then reading the next character in the receive FIFO.
#define FT_LSR_FIFO_DATA_ERROR 0x80 //this is just an indication that there is at least one parity error, framing error, or break indication in the receive FIFO.  This bit is cleared when there isn't any of the mentioned error.
#define FT_LSR_THR_TSR_ARE_EMPTY 0x40  //status of the transmitter hold and shift register
#define FT_LSR_THR_IS_EMPTY 0x20 //transmit holding register empty indicator.  
#define FT_LSR_BREAK_INTERRUPT 0x10  //break condition occurred (character 0x00)
#define FT_LSR_FRAMING_ERROR 0x08 //framing error in the top of the RX FIFO, received data did not have a valid stop bit.
#define FT_LSR_PARITY_ERROR 0x04 //parity error in RX FIFO
#define FT_LSR_OVERRUN_ERROR 0x02 //overrun in the RX FIFO has occurred
#define FT_LSR_DATA_IN_RECEIVER 0x01 //an indication that at least one character in the RX FIFO
#define FT_LSR_ALL_FIFO_ERRORS 0x1E //represents all the errors reported in the LSR register: break interrupt, framing error, parity error, and overrun error.

//The MSR register indicates the current state of the control lines from the modem, data set, or peripheral device to the host. The primary inputs RI, CD, CTS, DSR are all active LOW.
#define FT_MSR_CD_STATE 0x80 //
#define FT_MSR_RI_STATE 0x40
#define FT_MSR_DSR_STATE 0x20
#define FT_MSR_COMPLEMENT_OF_THE_CTS_INPUT 0x10 //active high, logical 1
#define FT_MSR_CD_CHANGED_STATE 0x08 //cleared on read
#define FT_MSR_RI_CHANGED_TO_HIGH 0x04 //cleared on read
#define FT_MSR_DSR_CHANGED_STATE 0x02 //clear on read
#define FT_MSR_CTS_CHANGED_STATE 0x01 //cleared on read

//The SPR register is used as a temporary data storage register.  The register is not cleared after a soft reset and it's only accessible when the MCR bit is not enabled for the TLR and TCR register.
#define FT_SPR_SCRATCH_PAD 0xff

//The TCR register controls the stop/start level used by the hardware/software flow control.  The valid values are from 0 to 60 with a granularity of four.  If either stop/start value is not set then the trigger level in TCR is used.  
#define FT_TCR_RX_RESUME_LEVEL 0xF0
#define FT_TCR_RX_HALT_LEVEL 0x0F

//The TLR register specifies the trigger level for interrupt generation.  Valid values are from 4 to 60 with a granularity of 4.
#define FT_TLR_RX_FIFO_TRIGGER_LEVEL 0xF0 //value represents the number of characters available
#define FT_TLR_TX_FIFO_TRIGGER_LEVEL 0x0F //value represents the number of spaces available.


//The TXLVL register reports the number of spaces available currently in the transmit FIFO
#define FT_TXLVL_TX_FIFO_FREE_SPACE 0x7F  //values are from 0 to 64

//The RXLVL register reports the fill level of the receive FIFO, the number of characters currently in the RX FIFO, there are 64 bytes.
#define FT_RXLVL_RX_FIFO_FILL_LEVEL 0x7F  //values from 0 to 64

//The IOControl register 
#define FT_IOCONTROL_SOFTWARE_RESET 0x08 //after the device resets itself, this bit will be automatically set to '0'
#define FT_IOCONTROL_SET_MODEM_PIN 0x02 //'0' = GPIO behave as I/O pins , '1' = GPIO behave as RI, CD, DTR, DSR
#define FT_IOCONTROL_ENABLE_INPUTS_LATCHING 0x01

//The EFCR register controls the extra features 
#define FT_EFCR_IRDA_MODE 0x80 //'0' = IrDA SIR 3/16 pulse ratio, data rate up to 115.2 kbit/s. '1' = IrDA SIR, 1/4 pulse ratio, data rate up to 1.152 Mbit/s
#define FT_EFCR_INVERT_RS485_RTS_SIGNAL 0x20
#define FT_EFCR_TRANSMITTER_CONTROSL_RTS 0x10
#define FT_EFCR_DISABLE_TRANSMITTER 0x04
#define FT_EFCR_DISABLE_RECEIVER 0x02
#define FT_EFCR_ENABLE_RS485_MODE 0x01 //0=rs232 mode , 1=rs485 mode
#define FT_EFCR_DISABLE_RS485_FEATURES 0x80


/*
LEGEND:

NOTE 1: These registers are accessible only when LCR[7] = 0.
NOTE 2: These bits in can only be modified if register bit EFR[4] is enabled.
NOTE 3: These bits are reserved and should be set to 0.
NOTE 4: After receive FIFO or transmit FIFO reset (through FCR[1:0]), the user must wait at least 2 x Tclk of XTAL1 before reading or writing data to RHR and THR, respectively.
NOTE 5: Burst reads on the serial interface (that is, reading multiple elements on the I2C-bus without a STOP or repeated START condition, or reading multiple elements on the SPI bus without de-asserting the CS pin), should not be performed on the IIR register.
NOTE 7: These registers are accessible only when MCR[2] = 1 and EFR[4] = 1.
NOTE 8: Device returns NACK on I2C-bus when this bit is written.
NOTE 10: The special register set is accessible only when LCR[7] = 1 and not 0xBF.
NOTE 11: Enhanced Feature Registers are only accessible when LCR = 0xBF.
*/

#endif /* _SC16IS760IBS_H_ */

